import sys

from pylons import config
from tg.controllers import TGController

class entry(object):
    def __init__(self, path, name, extension, permission, url):
        self._path = path
        self._mpath = [x.strip() for x in path.split('||')]
        self._name = name
        self._extension = extension
        self._permission = permission
        self._url = url
        self.func = None
        
    def __str__(self):
        return '%s at %s' % (str(self._path), str(self._url))

class shared_menu_cache(object):
    __shared_state = {}
    def __init__(self):
        self.__dict__ = self.__shared_state
        if not(hasattr(self, '_menuitems')):
            self._menuitems = {}
        if not(hasattr(self, 'needs_update')):
            self.needs_update = True

    def addEntry(self, menuitem):
        self.needs_update = True
        if menuitem._name not in self._menuitems:
            self._menuitems[menuitem._name] = {}
        self._menuitems[menuitem._name][menuitem._path] = menuitem
        
    def removeEntry(self, menuitem):
        if menuitem._name in self._menuitems:
            if menuitem._path in self._menuitems[menuitem._name]:
                del self._menuitems[menuitem._name][menuitem._path]
            
    def getEntry(self, menuname, path):
        if menuname in self._menuitems:
            if path in self._menuitems[menuname]:
                return self._menuitems[menuname][path]
            else:
                return None
        else:
            return None
        
        return None
    
    def removeMenu(self, menuname):
        if menuname in self._menuitems:
            del self._menuitems[menuname]
            
    def getMenu(self, menuname):
        if self.needs_update:
            self.updateUrls()
        return self._menuitems[menuname] if menuname in self._menuitems else dict()
    
    def updateUrls(self):
        pname = '%s.controllers.root' % (config['package'].__name__)
        __import__(pname)
        r = sys.modules[pname].RootController
        for menuname in self._menuitems:
            for menuitem in self._menuitems[menuname]:
                mi = self._menuitems[menuname][menuitem]
                mi._url = find_url(r, self._menuitems[menuname][menuitem])


def find_url(root, menuitem):
    n = menuitem.func.__name__
    if n in root.__dict__:
        if root.__dict__[n] is menuitem.func:
            return '/%s' % (n)
    for i in root.__dict__:
        controller = root.__dict__[i]
        if hasattr(controller, '_dispatch'):
            v = find_url(controller.__class__, menuitem)
            if v is not None:
                return '/%s%s' % (i, v)
    return None

shared_cache = shared_menu_cache()



from mako.template import Template

from pkg_resources import Requirement, resource_string
divmenu = Template(resource_string(Requirement.parse("tgext.menu"),"tgext/menu/templates/divmenu.mak"))

from pylons import config
from repoze.what.predicates import NotAuthorizedError
from repoze.what.predicates import has_permission
from tg import request
from tw.api import JSLink, CSSLink
from tw.jquery import jquery_js, jquery_bgiframe_js, jquery_dimensions_js

from caches import shared_cache

jquery_position_js = JSLink(
    modname=__name__,
    filename='static/jquery.positionBy.js',
    javascript=[jquery_js, jquery_bgiframe_js, jquery_dimensions_js])

jquery_jdmenu_js = JSLink(
    modname=__name__,
    filename='static/jquery.jdMenu.js',
    javascript=[jquery_js, jquery_bgiframe_js, jquery_dimensions_js, jquery_position_js])

jquery_jdmenu_css = CSSLink(modname=__name__, filename='static/jquery.jdMenu.css')

sortorder = {}

class OutputEntry(object):
    def __init__(self, name, href=None):
        self.children = []
        self.href = href
        self.name = name

    def appendPath(self, name, href):
        # name is a list of paths, i.e.: ['Foo Spot', 'Bar']
        if len(name) > 1:
            try:
                idx = self.children.index(name[0])
            except ValueError:
                child = OutputEntry(name[0])
                self.children.append(child)
                idx = len(self.children)-1
            self.children[idx].appendPath(name[1:], href)
        else:
            child = OutputEntry(name[0], href)
            self.children.append(child)
    
    def __eq__(self, othername):
        return self.name == othername
            
def url_from_menu():
    # @todo: make a function that will return the url for the given menu path
    raise NotImplementedError('url_from_menu: Not Yet Implemented')

def sort_entry(ent1, ent2):
    global sortorder
    idx = 0
    while idx < len(ent1._mpath) and idx < len(ent2._mpath):
        val1 = sortorder.get(ent1._mpath[idx], 999999)
        val2 = sortorder.get(ent2._mpath[idx], 999999)
        if val1 != val2:
            return cmp(val1, val2)
        idx += 1
    return cmp(ent1._mpath, ent2._mpath)

def permission_met(permission):
    if permission is None:
        return True
    elif type(permission) is str:
        try:
            has_permission(permission).check_authorization(request.environ)
            return True
        except NotAuthorizedError:
            return False
    else:
        try:
            permission.check_authorization(request.environ)
            return True
        except:
            return False
    
def render_menu(menuname, vertical=False):
    global sortorder
    jquery_js.inject()
    jquery_bgiframe_js.inject()
    jquery_dimensions_js.inject()
    jquery_position_js.inject()
    jquery_jdmenu_js.inject()
    
    if config.get('tgext_menu', {}).get('inject_css', False):
        jquery_jdmenu_css.inject()
    
    menutree = OutputEntry(menuname)
    menu = shared_cache.getMenu(menuname)
    sortorder = config.get('tgext_menu', {}).get('sortorder', {})
    shortmenu = [menu[key] for key in filter(lambda x: permission_met(menu[x]._permission), menu.keys())]
    for menuitem in sorted(shortmenu, sort_entry):
        menutree.appendPath(menuitem._mpath, str(menuitem._url))
    return divmenu.render(menulist=menutree, name=menuname, vertical_menu=vertical)

def render_navbar(vertical=False):
    return render_menu(u'navbar', vertical)

def render_sidebar(vertical=False):
    return render_menu(u'sidebar', vertical)

def render_sitemap(vertical=False):
    raise NotImplementedError('render_sitemap: Not Yet Implemented')
    return render_menu(u'sitemap', vertical)



from tg.configuration import Bunch
from pylons import config

from decorators import menu, navbar, sidebar, sitemap
from functions import render_menu, render_navbar, render_sidebar, render_sitemap, url_from_menu

__all__ = ["menu", "navbar", "sidebar", "sitemap", "url_from_menu"]

def menu_variable_provider():
    menu_vars = Bunch (
        url_from_menu = url_from_menu,
        render_menu = render_menu,
        render_navbar = render_navbar,
        render_sidebar = render_sidebar,
        render_sitemap = render_sitemap
    )
    
    if app_variable_provider:
        menu_vars.update(app_variable_provider())

    return menu_vars

app_variable_provider = config.get('variable_provider', None)
config.update(Bunch(
    variable_provider = menu_variable_provider
    ))



import sys
from caches import shared_cache, entry

class menu(object):
    def __init__(self, path, name, extension='html', permission=None, url=None):
        self.item = entry(path, name, extension, permission, url)
        
    def __call__(self, func):
        self.item.func = func
        shared_cache.addEntry(self.item)
        return func

class navbar(menu):
    def __init__(self, path, extension='html', permission=None, url=None):
        super(navbar, self).__init__(path, u'navbar', extension, permission, url)
    
class sidebar(menu):
    def __init__(self, path, extension='html', permission=None, url=None):
        super(sidebar, self).__init__(path, u'sidebar', extension, permission, url)

class sitemap(menu):
    def __init__(self, path, extension='html', permission=None, url=None):
        super(sidebar, self).__init__(path, u'sitemap', extension, permission, url)
